#include "04-NameReservation.hh"
#include "ReturnCodes.hh"

#include <RTI/certiRTI1516.h>
#include <RTI/Enums.h>
#include <iostream>
#include <sstream>
#include <stdlib.h>
#include <time.h>

int main(int argc, char** argv)
{
	int retCode1 = 0;

	srand( time(0) );

	TestFedAmb *fedAmb = new TestFedAmb();

	retCode1 = fedAmb->execute(argc, argv);



	if ( retCode1 == certi::rti1516_test::RETURN_NO_ERRORS)
	{
		std::wcerr << L"Pass: Test completed" << std::endl;
	} else
	{
		std::wcerr << L"Fail: Test failed with error code: " << retCode1 << std::endl;

	}

	// Poor man's 'pause'
	std::wcerr << L"Press enter to stop connection to RTI." << std::endl;
	getchar();

	delete fedAmb;

	// Poor man's 'pause'
	getchar();

	return retCode1;
}

TestFedAmb::TestFedAmb()
: rtiAmb(0)
, fedHandle()
, fedName()
, federationName(L"TestFederationName")
{
	int nameVal;

	nameVal = (rand() % 99) + 1;

	std::wstringstream stream;
	stream << L"TestFederate" << nameVal;

	fedName = stream.str();
	std::wcerr << L"Federate name:" << fedName << std::endl;

}

TestFedAmb::~TestFedAmb()
{
	delete rtiAmb;
}

int TestFedAmb::execute(int argc, char** argv)
{
	std::wcerr << L"01T Test: Attempting to set up test environment (create and join federation)" << std::endl;
	std::auto_ptr< rti1516::RTIambassadorFactory > rtiAmbFact(new rti1516::RTIambassadorFactory());
	std::vector< std::wstring > constructorArgs;
	try {
		//////////////////////////////////////////////
		// Set up
		std::auto_ptr< rti1516::RTIambassador > rtiAmbP(rtiAmbFact->createRTIambassador(constructorArgs));
		rtiAmb = rtiAmbP.release();

		try {
			rtiAmb->createFederationExecution(federationName, L"Certi-Test-02.xml");
			// expected
		} catch (rti1516::FederationExecutionAlreadyExists &)
		{
			// expected
		} catch (rti1516::Exception &)
		{
			throw;
		}

		fedHandle = rtiAmb->joinFederationExecution(fedName, federationName, *this);

	} catch (rti1516::Exception &e)
	{
		std::wcerr << L"01N Fail:" << e.what() << std::endl;
		return certi::rti1516_test::RETURN_ERROR_CREATING_AMBASSADOR;
	}

	std::wcerr << L"01E Pass: Federation joined. Press enter to continue with test." << std::endl;
	getchar();

	// CLASS HANDLE
	try {
		////////////////////////////////////////////////
		std::wcerr << L"02T Test: Attempting to get handle for invalid class" << std::endl;
		rti1516::ObjectClassHandle classHandle = rtiAmb->getObjectClassHandle(L"InvalidObjectClassName");
		////////////////////////////////////////////////
		std::wcerr << L"02N Fail: Could get class handle for invalid object class name" << std::endl;
		return certi::rti1516_test::RETURN_ERROR_GET_CLASS_HANDLE;
	} catch (rti1516::NameNotFound &e)
	{
		// expected
		std::wcerr << L"02E Pass:" << e.what() << std::endl;
	} catch (rti1516::Exception &e)
	{
		std::wcerr << L"02N Fail:" << e.what() << std::endl;
		return certi::rti1516_test::RETURN_ERROR_GET_CLASS_HANDLE;
	}

	try {
		////////////////////////////////////////////////
		std::wcerr << L"03T Test: Attempting to get handle for valid class name" << std::endl;
		rti1516::ObjectClassHandle classHandle = rtiAmb->getObjectClassHandle(L"TestObjectClass");
		////////////////////////////////////////////////
		// expected
		if ( classHandle.isValid() )
		{
			std::wcerr << L"03E Pass: Got class handle for valid class name" << std::endl;
		} else
		{
			std::wcerr << L"03N Fail: Got invalid class handle for valid class name" << std::endl;
			return certi::rti1516_test::RETURN_ERROR_GET_CLASS_HANDLE;
		}
	} catch (rti1516::Exception &e)
	{
		std::wcerr << L"03N Fail:" << e.what() << std::endl;
		return certi::rti1516_test::RETURN_ERROR_GET_CLASS_HANDLE;
	}

	// CLASS NAME
	try {
		////////////////////////////////////////////////
		std::wcerr << L"04T Test: Attempting to get name for invalid class handle" << std::endl;
		rti1516::ObjectClassHandle classHandle;
		std::wstring className = rtiAmb->getObjectClassName(classHandle);
		////////////////////////////////////////////////
		std::wcerr << L"04N Fail: Could get class name for invalid class handle. Name returned:" << className << std::endl;
		return certi::rti1516_test::RETURN_ERROR_GET_CLASS_NAME;
	} catch (rti1516::InvalidObjectClassHandle &e)
	{
		// expected
		std::wcerr << L"04E Pass: " << e.what() << std::endl;
	} catch (rti1516::Exception &e)
	{
		std::wcerr << L"04N Fail: " << e.what() << std::endl;
		return certi::rti1516_test::RETURN_ERROR_GET_CLASS_NAME;
	}

	try {
		////////////////////////////////////////////////
		std::wcerr << L"05T Test: Attempting to get name for valid class handle" << std::endl;
		std::wstring className(L"TestObjectClass");
		rti1516::ObjectClassHandle classHandle = rtiAmb->getObjectClassHandle(className);
		std::wstring retClassName = rtiAmb->getObjectClassName(classHandle);
		////////////////////////////////////////////////
		// expected
		if ( className != retClassName )
		{
			std::wcerr << L"05N Fail: Returned classname does not match expected classname. " << 
						  L"(Expected " << className << L", got " << retClassName << L".)" << std::endl;
			return certi::rti1516_test::RETURN_ERROR_GET_CLASS_NAME;
		} else 
		{
			std::wcerr << L"05E Pass: Got expected classname" << std::endl;
		}
	} catch (rti1516::Exception &e)
	{
		std::wcerr << L"05N Fail: " << e.what() << std::endl;
		return certi::rti1516_test::RETURN_ERROR_GET_CLASS_NAME;
	}

	// ATTRIBUTE HANDLE
	try {
		////////////////////////////////////////////////
		std::wcerr << L"06T Test: Attempting to get attribute handle for invalid class" << std::endl;
		rti1516::ObjectClassHandle classHandle;
		rti1516::AttributeHandle attrHandle = rtiAmb->getAttributeHandle(classHandle, L"InvalidAttributeName");
		////////////////////////////////////////////////
		std::wcerr << L"06N Fail: Could get attribute handle for invalid class handle" << std::endl;
		return certi::rti1516_test::RETURN_ERROR_GET_ATTRIBUTE_HANDLE;
	} catch (rti1516::InvalidObjectClassHandle &e)
	{
		// expected
		std::wcerr << L"06E Pass:" << e.what() << std::endl;
	} catch (rti1516::Exception &e)
	{
		std::wcerr << L"06N Fail:" << e.what() << std::endl;
		return certi::rti1516_test::RETURN_ERROR_GET_ATTRIBUTE_HANDLE;
	}

	try {
		////////////////////////////////////////////////
		std::wcerr << L"07T Test: Attempting to get handle for invalid attribute name" << std::endl;
		rti1516::ObjectClassHandle classHandle = rtiAmb->getObjectClassHandle(L"TestObjectClass");
		rti1516::AttributeHandle attrHandle = rtiAmb->getAttributeHandle(classHandle, L"InvalidAttributeName");
		////////////////////////////////////////////////
		std::wcerr << L"07N Fail: Could get attribute handle for invalid attribute name" << std::endl;
		return certi::rti1516_test::RETURN_ERROR_GET_ATTRIBUTE_HANDLE;
	} catch (rti1516::NameNotFound &e)
	{
		// expected
		std::wcerr << L"07E Pass:" << e.what() << std::endl;
	} catch (rti1516::Exception &e)
	{
		std::wcerr << L"07N Fail:" << e.what() << std::endl;
		return certi::rti1516_test::RETURN_ERROR_GET_ATTRIBUTE_HANDLE;
	}

	try {
		////////////////////////////////////////////////
		std::wcerr << L"08T Test: Attempting to get handle for valid attribute class" << std::endl;
		rti1516::ObjectClassHandle classHandle = rtiAmb->getObjectClassHandle(L"TestObjectClass");
		rti1516::AttributeHandle attrHandle = rtiAmb->getAttributeHandle(classHandle, L"Data");
		////////////////////////////////////////////////
		// expected
		if ( classHandle.isValid() )
		{
			std::wcerr << L"08E Pass: Got attribute handle for valid attribute class & name" << std::endl;
		} else
		{
			std::wcerr << L"08N Fail: Got invalid class handle for valid class name" << std::endl;
			return certi::rti1516_test::RETURN_ERROR_GET_ATTRIBUTE_HANDLE;
		}
	} catch (rti1516::Exception &e)
	{
		std::wcerr << L"08N Fail:" << e.what() << std::endl;
		return certi::rti1516_test::RETURN_ERROR_GET_ATTRIBUTE_HANDLE;
	}

	// ATTRIBUTE NAME
	std::wcerr << L"09T Test: Attempting to get name for invalid attribute handle" << std::endl;
	std::wcerr << L"09E Skip:" << std::endl;
	std::wcerr << L"10T Test: Attempting to get name for valid attribute handle" << std::endl;
	std::wcerr << L"10E Skip:" << std::endl;

	// INTERACTION HANDLE
	try {
		////////////////////////////////////////////////
		std::wcerr << L"11T Test: Attempting to get handle for invalid interaction" << std::endl;
		rti1516::InteractionClassHandle msgHandle = rtiAmb->getInteractionClassHandle(L"InvalidInteractionName");
		////////////////////////////////////////////////
		std::wcerr << L"11N Fail: Could get handle for invalid interaction name." << std::endl;
		return certi::rti1516_test::RETURN_ERROR_GET_INTERACTION_HANDLE;
	} catch (rti1516::NameNotFound &e)
	{
		// expected
		std::wcerr << L"11E Pass: " << e.what() << std::endl;
	} catch (rti1516::Exception &e)
	{
		std::wcerr << L"11N Fail: " << e.what() << std::endl;
		return certi::rti1516_test::RETURN_ERROR_GET_INTERACTION_HANDLE;
	}

	try {
		////////////////////////////////////////////////
		std::wcerr << L"12T Test: Attempting to get handle for valid interaction name" << std::endl;
		rti1516::InteractionClassHandle msgHandle = rtiAmb->getInteractionClassHandle(L"Msg");
		////////////////////////////////////////////////
		// expected
        if (msgHandle.isValid() )
        {
			std::wcerr << L"12E Pass: Got interaction handle for valid interaction name" << std::endl;
		} else
		{
			std::wcerr << L"12N Fail: Got invalid interaction handle for valid interaction name" << std::endl;
			return certi::rti1516_test::RETURN_ERROR_GET_INTERACTION_HANDLE;
        }
	} catch (rti1516::Exception &e)
	{
		std::wcerr << L"12N Fail:" << e.what() << std::endl;
		return certi::rti1516_test::RETURN_ERROR_GET_INTERACTION_HANDLE;
	}

	// INTERACTION NAME
	std::wcerr << L"13T Test: Attempting to get name for invalid interaction handle" << std::endl;
	std::wcerr << L"13E Skip:" << std::endl;
	std::wcerr << L"14T Test: Attempting to get name for valid interaction handle" << std::endl;
	std::wcerr << L"14E Skip:" << std::endl;

	// PARAMETER HANDLE
	std::wcerr << L"15T Test: Attempting to get handle for parameter class" << std::endl;
	std::wcerr << L"15E Skip:" << std::endl;
	std::wcerr << L"16T Test: Attempting to get handle for parameter class" << std::endl;
	std::wcerr << L"16E Skip:" << std::endl;

	// PARAMETER NAME
	std::wcerr << L"16T Test: Attempting to get name for invalid parameter handle" << std::endl;
	std::wcerr << L"16E Skip:" << std::endl;
	std::wcerr << L"17T Test: Attempting to get name for valid parameter handle" << std::endl;
	std::wcerr << L"17E Skip:" << std::endl;

	// OBJECT HANDLE ?
	std::wcerr << L"18T Test: Attempting to get handle for object class" << std::endl;
	std::wcerr << L"18E Skip:" << std::endl;
	std::wcerr << L"19T Test: Attempting to get handle for object class" << std::endl;
	std::wcerr << L"19E Skip:" << std::endl;

	// OBJECT NAME ?
	std::wcerr << L"20T Test: Attempting to get name for invalid object handle" << std::endl;
	std::wcerr << L"20E Skip:" << std::endl;
	std::wcerr << L"21T Test: Attempting to get name for valid object handle" << std::endl;
	std::wcerr << L"21E Skip:" << std::endl;

	std::wcerr << L"Test operations performed. Press enter to continue with shutdown." << std::endl;
	getchar();

	try {
		////////////////////////////////////////////////
		std::wcerr << L"Attempting to resign from Federation" << std::endl;
		rtiAmb->resignFederationExecution(rti1516::CANCEL_THEN_DELETE_THEN_DIVEST);
		std::wcerr << L"Resigned from Federation" << std::endl;
		std::wcerr << L"Attempting to destroy Federation" << std::endl;
		rtiAmb->destroyFederationExecution(L"TestFederationName");
		std::wcerr << L"Destroyed Federation" << std::endl;
		////////////////////////////////////////////////
		// expected
	} catch (rti1516::FederatesCurrentlyJoined &)
	{
		// expected
		std::wcerr << L"Could not destroy federation while another federate was still connected. This is expected behaviour." << std::endl;
	} catch (rti1516::Exception &e)
	{
		std::wcerr << L"17N " << e.what() << std::endl;
		return certi::rti1516_test::RETURN_ERROR_DESTROYING_FEDERATION;
	}

	return certi::rti1516_test::RETURN_NO_ERRORS;
}